@@ -4,9 +4,11 @@
 REST functionality based off pycurlbrowser's Browser.
 """
 
-from pycurlbrowser import Browser
-import pycurl
-import simplejson as json
+try:
+    import simplejson as json
+except ImportError:
+    import json
+from . import Browser
 
 class StatusInformational(Exception):
 
@@ -52,49 +54,52 @@ class RestClient(Browser):
     A simple REST client based upon pycurlbrowser
     """
 
-    def __init__(self, base):
-        super(RestClient, self).__init__()
-        self._curl.setopt(pycurl.USERAGENT, "pycurl.rest_client 0.1")
+    def __init__(self, base, *args, **kwargs):
+        super(RestClient, self).__init__(*args, **kwargs)
         self.base = base
 
-    def go(self, obj, method, uid=None, data=None):
+    def go(self, obj, method, uid=None, data=None, headers=None):
         url = '%(base)s/%(obj)s' % {'base': self.base,
                                     'obj' : obj}
         if uid is not None:
             url += '/%s' % uid
 
-        res = super(RestClient, self).go(url, method, data, escaped=True)
-        if res != 200:
-            raise status_factory(res)
-        return res
+        super(RestClient, self).go(url=url,
+                                   method=method,
+                                   data=data,
+                                   headers=headers)
+        if self.http_code != 200:
+            raise status_factory(self.http_code)
+
+        return self.http_code
 
     # CRUD
 
-    def create(self, obj, data=None):
+    def create(self, obj, data=None, headers=None):
         """Create (POST)"""
-        self.go(obj, 'POST', data=data)
+        self.go(obj, 'POST', data=data, headers=headers)
         return self.src
 
-    def read(self, obj, uid=None):
+    def read(self, obj, uid=None, headers=None):
         """Read (GET)"""
-        self.go(obj, 'GET', uid=uid)
+        self.go(obj, 'GET', uid=uid, headers=headers)
         return self.src
 
-    def head(self, obj, uid=None):
+    def head(self, obj, uid=None, headers=None):
         """Head (HEAD)"""
         # TODO: care about headers
-        self._curl.setopt(pycurl.NOBODY, 1)
-        self.go(obj, 'HEAD', uid=uid)
+        # TODO: think about self._curl.setopt(pycurl.NOBODY, 1)
+        self.go(obj, 'HEAD', uid=uid, headers=headers)
 
-    def update(self, obj, uid, data=None):
+    def update(self, obj, uid, data=None, headers=None):
         """Update (PUT)"""
-        self.go(obj, 'PUT', uid=uid, data=data)
+        self.go(obj, 'PUT', uid=uid, data=data, headers=headers)
         return self.src
 
-    def destroy(self, obj, uid):
+    def destroy(self, obj, uid, headers=None):
         """Destroy (DELETE)"""
         # TODO: care about headers
-        self.go(obj, 'DELETE', uid=uid)
+        self.go(obj, 'DELETE', uid=uid, headers=headers)
         return self.src
 
 class RestClientJson(RestClient):
@@ -105,8 +110,7 @@ class RestClientJson(RestClient):
 
     def create(self, obj, data=None):
         """Create (POST)"""
-        self._curl.setopt(pycurl.HTTPHEADER, ['Content-Type: text/json'])
-        res = super(RestClientJson, self).create(obj, json.dumps(data))
+        res = super(RestClientJson, self).create(obj, json.dumps(data), headers={'Content-Type': 'text/json'})
         if len(res) > 0:
             return json.loads(res)
         return None
@@ -117,8 +121,7 @@ class RestClientJson(RestClient):
 
     def update(self, obj, uid, data=None):
         """Update (PUT)"""
-        self._curl.setopt(pycurl.HTTPHEADER, ['Content-Type: text/json'])
-        res = super(RestClientJson, self).update(obj, uid, json.dumps(data))
+        res = super(RestClientJson, self).update(obj, uid, json.dumps(data), headers={'Content-Type': 'text/json'})
         if len(res) > 0:
             return json.loads(res)
         return None
